local MAXMOVE = 60*FRACUNIT/NEWTICRATERATIO

local function P_FDLightDash(source, enemy) // Home in on your target
	if not (source and source.valid
	and enemy and enemy.valid)
		return
	end
	
	local dist
	local dest
	
	if not (source.tracer and source.tracer.valid)
		return // Nothing to home in on!
	end
	
	// adjust direction
	dest = source.tracer
	
	if not dest
		return
	end
	
	// change angle
	source.angle = R_PointToAngle2(source.x, source.y, enemy.x, enemy.y)
	if source.player and source.player.valid
		source.player.drawangle = source.angle
	end
	
	// change slope
	dist = P_AproxDistance(P_AproxDistance(dest.x - source.x, dest.y - source.y), dest.z - source.z)
	
	if dist < 1
		dist = 1
	end
	
	local playerdistx = FixedDiv((dest.x - source.x), dist)
	local playerdisty = FixedDiv((dest.y - source.y), dist)
	local playerdistz = FixedDiv((dest.z - source.z), dist)
	
	local maxMoveScaled = FixedMul(MAXMOVE, source.scale)
	
	source.momx = FixedTrunc(FixedMul(playerdistx, maxMoveScaled))
	source.momy = FixedTrunc(FixedMul(playerdisty, maxMoveScaled))
	source.momz = FixedTrunc(FixedMul(playerdistz, maxMoveScaled))
end

local function P_LookForRings(player)
	local found = false
	
	player.mo.target = nil
	player.mo.tracer = nil
	
	searchBlockmap("objects", function(pmo, mo)
		if mo.health <= 0 or mo.state == S_NULL
			return
		end
		
		if not (mo.type == MT_RING or mo.type == MT_COIN or mo.info.fdadvRing)
			return
		end
		
		if P_AproxDistance(P_AproxDistance(pmo.x-mo.x, pmo.y-mo.y),
			pmo.z-mo.z) > 192*pmo.scale
			player.canlightdash = true	
		end
		
		if not P_CheckSight(pmo, mo)
			return
		end
		
		if pmo.target and P_AproxDistance(P_AproxDistance(pmo.x-mo.x,
			pmo.y-mo.y), pmo.z-mo.z) > 
			P_AproxDistance(P_AproxDistance(pmo.x-pmo.target.x,
			pmo.y-pmo.target.y), pmo.z-pmo.target.z)
			return
		end
		
		found = true
		pmo.target = mo
		pmo.tracer = mo
	end, player.mo,
	player.mo.x - 192*player.mo.scale, player.mo.x + 192*player.mo.scale,
	player.mo.y - 192*player.mo.scale, player.mo.y + 192*player.mo.scale)
	
	if found
		P_ResetPlayer(player)
		player.fdhoming = 0
		player.mo.state = S_PLAY_FALL
		P_ResetScore(player)
		P_FDLightDash(player.mo, player.mo.target)
		return
	end
	player.mo.momx = $/2
	player.mo.momy = $/2
	player.mo.momz = $/2
	player.fdlightdash = false	
end

local function P_RingNearby(player) 
	local closest = nil
	
	searchBlockmap("objects", function(pmo, mo)
		if mo.health <= 0 or mo.state == S_NULL
			return
		end
		
		if not (mo.type == MT_RING or mo.type == MT_COIN or mo.info.fdadvRing)
			return
		end
		
		if P_AproxDistance(P_AproxDistance(pmo.x-mo.x, pmo.y-mo.y),
			pmo.z-mo.z) > 192*pmo.scale // Out of range
			return
		end
				
		if not P_CheckSight(pmo, mo) // Out of sight
			return
		end
		
		if closest and P_AproxDistance(P_AproxDistance(pmo.x-mo.x, pmo.y-mo.y),
			pmo.z-mo.z) > P_AproxDistance(P_AproxDistance(pmo.x-closest.x,
			pmo.y-closest.y), pmo.z-closest.z)
			return
		end
		
		// Found a target
		closest = mo
	end, player.mo,
	player.mo.x - 192*player.mo.scale, player.mo.x + 192*player.mo.scale,
	player.mo.y - 192*player.mo.scale, player.mo.y + 192*player.mo.scale)	
	
	if closest
		return true
	end
	
	return false
end

addHook("PlayerThink", function(player)
	if player.mo.skin == "fdsonic" and (player.cmd.buttons & BT_CUSTOM2) then
		if P_RingNearby(player) then
			if player.mo.state != S_PLAY_DEAD or player.mo.state != S_PLAY_HURT then
				player.fdlightdash = TICRATE
			end
		end
	end
	if player.fdlightdash > 0 then
		P_LookForRings(player)
		player.fdlightdash = player.fdlightdash - 1
	end
	if not P_RingNearby(player) then
		player.fdlightdash = 0
		player.canlightdash = false
	end
	if player.mo.state == S_PLAY_HURT then
		player.fdlightdash = 0
	end
end)

hud.add(function(v, player)
if player.mo.skin == "fdsonic"
local activateText = "CUSTOM2:"
local lightdashText = "LIGHT DASH"
v.drawFill(224, 6, 72, 12, 103|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER) // Color was originally 119
v.drawString(260, 8, activateText, V_TRANSLUCENT|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER, "center")
v.drawString(260, 20, lightdashText, V_TRANSLUCENT|V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER, "center")
end
end, "game")